@@ -8,6 +8,8 @@ from .client import JellyfinClient
 
 #################################################################################################
 
+__version__ = '1.11.0'
+
 
 class NullHandler(logging.Handler):
     def emit(self, record):
@@ -27,11 +29,8 @@ def config(level=logging.INFO):
     logger.setLevel(level)
 
 def has_attribute(obj, name):
-    try:
-        object.__getattribute__(obj, name)
-        return True
-    except AttributeError:
-        return False
+    return hasattr(obj, name)
+
 
 def ensure_client():
 
